import logging
import re

from bandersnatch.filter import FilterProjectPlugin, FilterReleasePlugin

logger = logging.getLogger("bandersnatch")


class RegexReleaseFilter(FilterReleasePlugin):
    """
    Filters releases based on regex patters defined by the user.
    """

    name = "regex_release"

    def initialize_plugin(self):
        """
        Initialize the plugin reading patterns from the config.
        """
        # TODO: should retrieving the plugin's config be part of the base class?
        try:
            config = self.configuration["filter_regex"]["releases"]
        except KeyError:
            self.patterns = []
        else:
            pattern_strings = [pattern for pattern in config.split("\n") if pattern]
            self.patterns = [
                re.compile(pattern_string) for pattern_string in pattern_strings
            ]

            logger.info(f"Initialized regex release plugin with {self.patterns}")

    def check_match(self, name, version):
        """
        Check if a release version matches any of the specificed patterns.

        Parameters
        ==========
        name: str
            Release name
        version: str
            Release version

        Returns
        =======
        bool:
            True if it matches, False otherwise.
        """
        return any(pattern.match(version) for pattern in self.patterns)


class RegexProjectFilter(FilterProjectPlugin):
    """
    Filters projects based on regex patters defined by the user.
    """

    name = "regex_project"

    def initialize_plugin(self):
        """
        Initialize the plugin reading patterns from the config.
        """
        try:
            config = self.configuration["filter_regex"]["packages"]
        except KeyError:
            self.patterns = []
        else:
            pattern_strings = [pattern for pattern in config.split("\n") if pattern]
            self.patterns = [
                re.compile(pattern_string) for pattern_string in pattern_strings
            ]

            logger.info(f"Initialized regex release plugin with {self.patterns}")

    def check_match(self, name):
        """
        Check if a release version matches any of the specificed patterns.

        Parameters
        ==========
        name: str
            Release name

        Returns
        =======
        bool:
            True if it matches, False otherwise.
        """
        return any(pattern.match(name) for pattern in self.patterns)



#!/usr/bin/env python3
from typing import NamedTuple


class _VersionInfo(NamedTuple):
    major: int
    minor: int
    micro: int
    releaselevel: str
    serial: int

    @property
    def version_str(self) -> str:
        release_level = f".{self.releaselevel}" if self.releaselevel else ""
        return f"{self.major}.{self.minor}.{self.micro}{release_level}"


__version_info__ = _VersionInfo(
    major=3,
    minor=1,
    micro=2,
    releaselevel="",
    serial=0,  # Not currently in use with Bandersnatch versioning
)
__version__ = __version_info__.version_str



import os.path

import pkg_resources


class Requirements(object):
    """Generate a pip compatible requirements.txt from our working set based on
    the 'bandersnatch' requirement."""

    def __init__(self, buildout, name, options) -> None:
        self.name, self.options = name, options
        options["path"] = os.path.join(
            buildout["buildout"]["directory"], "requirements.txt"
        )

    def install(self):
        path = self.options["path"]
        require = pkg_resources.Requirement.parse("bandersnatch")
        dependencies = pkg_resources.working_set.resolve([require])
        seen = set()
        with open(path, "w") as requirements_txt:
            requirements_txt.write("# Generated by buildout\n")
            for dep in reversed(dependencies):
                dep = dep.as_requirement()
                if dep.key in seen:
                    continue
                seen.add(dep.key)
                requirements_txt.write(str(dep) + "\n")
        return path

    update = install
